#include <Application.h>
#include <Button.h>
#include <Entry.h>
#include <Path.h>
#include <Roster.h>
#include <String.h>
#include <StringView.h>
#include <TextControl.h>
#include <View.h>
#include <Window.h>

#include <iostream>
using namespace std;
#include "PlayLog.h"
#include "Playlist.h"

//message constants
#define DESTINATION_BROWSE 'brws'
#define CONFIG_OK 'cfok'
#define CONFIG_CANCEL 'cfcn'


void *getplugin(void **data,const char *name, const char *, uint32, plugin_info * );
void destroyplugin(void*,void *_loader);
BView* configure(BMessage* configData);
void setconfig(void*,BMessage *config);
void getconfig(void*,BMessage *config);

class ConfigureView : public BView
{
public:
	ConfigureView(BMessage* msg);
	~ConfigureView();
	void AttachedToWindow();
	void MessageReceived(BMessage* msg);
	
private:
	BMessage*	fData;
	BTextControl	*fMaxSongs_TC, *fDestination_TC;
	BFilePanel* fFilePanel;
	BButton* fOkBtn, *fCancelBtn, *fBrowseBtn;
	
};


const char *PLUGINID="nhs-PlayLog";

static plugin_descriptor plugin={
	PLUGIN_DESCRIPTOR_MAGIC,PLUGIN_DESCRIPTOR_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,

	"PlayLog",
	"Dumps a log of your most recently played songs in BMessage format; great for Stupid Blog Tricks.\n\nBased on the ""EventLogger"" plugin by Marco Nelissen.\n\nModified by Michael Armida.\ninfo@nerdherdsoftware.com",
	NULL,
	configure,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_INTERFACE_MAGIC, PLUGIN_INTERFACE_VERSION,
	NULL,
	NULL,
	getconfig, //aren't these backwards?!!  cout'ing shows that the API docs have it reversed; the correct order is used here.
	setconfig
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}

void setconfig(void * data,BMessage *config)
{
	((PLLooper*)data) -> SaveSettings(config);
}

void getconfig(void* data,BMessage *config)
{
	((PLLooper*)data) -> LoadSettings(config);
}

BView* configure(BMessage* configData)
{
	ConfigureView* view = new ConfigureView(configData);
	return view;
}

ConfigureView::ConfigureView(BMessage* configData)
	: BView(BRect(0,0,244,147), "config_background", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW), 
	fData(configData),
	fFilePanel(new BFilePanel(B_OPEN_PANEL, &BMessenger(this), NULL, B_FILE_NODE | B_SYMLINK_NODE, false, NULL, NULL, false, true))
{	
	SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	fMaxSongs_TC = new BTextControl( BRect(8, 9, 184, 28), "maxsongs_tc", "Maximum number of songs: ", NULL, NULL);
	fDestination_TC = new BTextControl( BRect(8, 66, 238, 85), "destination_tc", NULL, NULL, NULL);
	fMaxSongs_TC->SetDivider(150);
	
	//load up data from the message
	int32 maxsongs;
	entry_ref destref;
	if( configData->FindRef("destination", &destref) != B_OK )
	{
		BEntry entry("/boot/home/PlayLog-output");
		entry.GetRef(&destref);
	}
	if( configData->FindInt32("maxsongs", &maxsongs) != B_OK )
	{
		maxsongs = DEFAULT_MAX_SONGS;
	}
	
	BString tempstring; //itoa(...) wasn't available, and I don't wanna do this the hard way.
	tempstring<<maxsongs;
	fMaxSongs_TC->SetText( tempstring.String() );
	BPath temppath( &destref );
	fDestination_TC->SetText( temppath.Path() );
	
	AddChild(fMaxSongs_TC);
	AddChild(fDestination_TC);
	
	BStringView* destlabel = new BStringView(BRect(10, 48, 151, 61), "destlabel", "Output file destination" );
	AddChild(destlabel);

	fBrowseBtn = new BButton( BRect(161, 40, 184, 55), "nhspl:browsebutton", "...", new BMessage(DESTINATION_BROWSE));
	AddChild(fBrowseBtn);
	
	fOkBtn = new BButton( BRect(197, 97, 238, 119), "nhspl:okbutton", "Ok", new BMessage(CONFIG_OK));
	fOkBtn->MakeDefault(true);
	AddChild(fOkBtn);
	fCancelBtn = new BButton( BRect(134, 98, 174, 118), "nhspl:cancelbutton", "Cancel", new BMessage(CONFIG_CANCEL));
	AddChild(fCancelBtn);	
}

void
ConfigureView::AttachedToWindow()
{	
	BMessenger invokerTarget(this, NULL);
	
	fFilePanel->SetTarget(invokerTarget);
	
	fOkBtn->SetTarget(invokerTarget);
	fCancelBtn->SetTarget(invokerTarget);
	fBrowseBtn->SetTarget(invokerTarget);
}

ConfigureView::~ConfigureView()
{
	delete fFilePanel;
}

void
ConfigureView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
		case CONFIG_CANCEL:
			Window()->PostMessage(B_QUIT_REQUESTED); //I don't have to do anything to SP to close the config window, do I?
			break;
			
		case CONFIG_OK:
		{
			//save values in views back to message pointed to by fData
			fData->MakeEmpty();
			fData->AddInt32( "maxsongs", atoi( fMaxSongs_TC->Text() ) );
			entry_ref ref;
			BEntry tempentry( fDestination_TC->Text() );
			tempentry.GetRef(&ref);
			fData->AddRef( "destination", &ref );
			
			//close the window
			Window()->PostMessage(B_QUIT_REQUESTED);
		}
			break;
			
		case B_REFS_RECEIVED:
		{
			entry_ref ref;
			msg->FindRef("refs", &ref);
			BEntry tempEntry (&ref);
			BPath tempPath( &tempEntry );
			fDestination_TC->SetText( tempPath.Path() );
		}
		
			break;
			
		case DESTINATION_BROWSE:
			fFilePanel->Show();
			break;	
			
		default:
			BView::MessageReceived(msg);	
	}	
}

void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info )
{
	PLLooper *looper=new PLLooper(info->controller);
	looper->Run();
	*data=looper;
	return &plugin_ops;
}

void destroyplugin(void *,void *data)
{
	PLLooper* looper=(PLLooper*)data;
	looper->PostMessage(B_QUIT_REQUESTED);
}

PLLooper::PLLooper(SoundPlayController *controller)
	: BLooper("PlayLog PLLooper")
{
	ctrl=controller;
	
	ctrl->Lock();
//	LogMessage("Starting up...");
	ctrl->AddListener(this);
	for(int i=0;;i++)
	{
		PlaylistPtr playlist=ctrl->PlaylistAt(i);
		if(!playlist->Lock())
			break;
//		LogMessage("Found track");
		playlist->AddListener(this);
		playlist->Unlock();
	}
//	LogMessage("Listening");
	ctrl->Unlock();
}

PLLooper::~PLLooper()
{
}

void 
PLLooper::LogMessage(const char* message)
{
	if( fDataFile.SetTo(&fRef, B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE) == B_OK )
	{
		while( fList.size() >= fMaxSongs )
			fList.pop_back();
		fList.push_front(BString(message)); //is this still o(1)?
	
		BMessage msg;
		for(list<BString>::iterator iter = fList.begin(); 
			iter != fList.end();
			iter++)
		{
			msg.AddString( "name", *iter );
		}
		
		msg.Flatten( &fDataFile );
	}
}

void PLLooper::MessageReceived(BMessage *mes)
{
	// re-add the playlists to the window in case they got orphaned
	switch(mes->what)
	{
		case PLAYLIST_CURRENT:
			{
				ctrl->Lock();
				
				//int32 newindex=mes->FindInt32("ID");
				PlaylistPtr playlist = ctrl->PlaylistAt(0);
				playlist->Lock();
				LogMessage(playlist->CurrentName());
				playlist->Unlock();
				ctrl->Unlock();
			}
			break;

		default:
			BLooper::MessageReceived(mes);
			break;
	}
}

bool PLLooper::QuitRequested()
{
	ctrl->DisableInterface(PLUGINID);
	return false;
}

void PLLooper::LoadSettings(BMessage* msg)
{
	if( msg->FindRef("destination", &fRef) != B_OK )
	{
		BEntry entry("/boot/home/PlayLog-output");
		entry.GetRef(&fRef);
	}
	
	if( msg->FindInt32("maxsongs", &fMaxSongs) != B_OK )
	{
		fMaxSongs = DEFAULT_MAX_SONGS;
	}
	
	if( fDataFile.SetTo(&fRef, B_READ_ONLY ) == B_OK )
	{
		BMessage data;
		if( data.Unflatten(&fDataFile) == B_OK )
		{
			int32 fields;
			type_code type;
			if( data.GetInfo("name", &type, &fields) == B_OK )
			{
				int32 max = (fields<fMaxSongs ? fields : fMaxSongs);
				for(int32 ctr=0; ctr<max; ctr++)
				{
					const char* songname;
					if( data.FindString("name", ctr, &songname) == B_OK)
						fList.push_back(songname);
				}
			}
		}
	}
}

void PLLooper::SaveSettings(BMessage* msg)
{
	msg->MakeEmpty();
	
	msg->AddRef("destination", &fRef);
	msg->AddInt32("maxsongs", fMaxSongs);
}

int main()
{
	BApplication app("application/x-vnd.marcone-spmixer");
	
	BMessage msg(ENABLE_INTERFACE);
	msg.AddString("interface",PLUGINID);
	be_roster->Launch("application/x-vnd.marcone-soundplay",&msg);
	return 0;
}

